%% FP cavity to study performance using various map combination

%{
%examples
% LLO X arm cold
fp = FPwMaps('ITMID', 4, 'ETMID', 10);
fp.lock;
fp.roundtripLoss
HOM0 = fp.LGFrac('ETM-HR-o', -10, -1);

% point absorber at (2cm,0) 30mW
fp = FPwMaps('ITMID', 4, 'ETMID', 10, 'xPoE', 0.02, 'pointPwrE', 0.03); 
fp.lock;
fp.roundtripLoss
HOM1 = fp.LGFrac('ETM-HR-o', -10, -1);

% two point absorbers at (2cm,0) and (-2cm,0) 30mW
fp = FPwMaps('ITMID', 4, 'ETMID', 10, 'xPoE', [0.02, -0.02], 'pointPwrE', [0.03, 0.03]); 
fp.lock;
fp.roundtripLoss
HOM2 = fp.LGFrac('ETM-HR-o', -10, -1);
figure(1)
semilogy( 0:11, [HOM0; HOM1; HOM2] )
xlabel('Modes'); ylabel('Mode fraction');

% two point absorers and coating absorption heating
fp = FPwMaps('ITMID', 4, 'ETMID', 10, 'xPoE', [0.02, -0.02], 'pointPwrE', [0.03, 0.03], 'PcoatE', 400e3*0.5e-6); 
fp.lock;
fp.roundtripLoss
[HRmap, xMap] = fp.optics.getHRmap('ETM');
figure(2)
plot( xMap, HRmap(end/2,:)*1e9)
axis([-0.15, 0.15, -10, 15])

% now BRDF
% to cover angle up to 5e-4, we need spatial resolution of 1e-6/5e-4 = 2mm and final window up to 4000*5e-4 = 2m
fp = FPwMaps('resolution', 0.001);
fp.lock;
% field going to ETM
[EE, xE] = fp.getField('ETM-HR-i', 2);  % 2 means the area of window of the field
[EI, xI] = fp.getField('ITM-HR-i', 2);
[BRDFE, theE] = BRDF( EE, xE, 4000, 5e-4);
[BRDFI, theI] = BRDF( EI, xI, 4000, 5e-4);
figure(3)
semilogy( theE, BRDFE, theI, BRDFI)
%}

classdef FPwMaps < FFTIFOAcc
    
    methods
        function obj = FPwMaps( varargin )
            obj@FFTIFOAcc( varargin{:} );
        end

        function defineIFO( obj, varargin )
            % default values of cavity parameters, which can be changed

            Lcav = 3994.5;
            ITMAper = 0.326;
            ETMAper = 0.326;
            
            ITMID = -4; ETMID = -4; % -4 is LLO Y arm
            
            eval( UtilTK.setVars( {'ITMID', 'ETMID'}, true, varargin{:} ) );

            % get the test mass information
            [ITMmap, ITMRoC, T_ITM] = TMInfo('I', ITMID, './Data0/');
            [ETMmap, ETMRoC, T_ETM] = TMInfo('E', ETMID, './Data0/');

            resolution = 0;            
            
            % God's RH
            removeCurvature = false;
            
            % for ITM surface
            delRoCI = 0;   % RH curvature correction
            PcoatI = 0;    % coating power absorption
            xPoI = 0; yPoI = 0; pointPwrI = 0; % point absorber location and power
            % for ETM
            delRoCE = 0; PcoatE = 0; xPoE = 0; yPoE = 0; pointPwrE = 0;
                
            % make the total RTL to be 65ppm
            RTL = 50e-6; 
            
            % poorman's WFS
            removeTilt = true;
            
            % parameters can be changed by runtime arguments, like fp = FPIFO('tiltE', 1e-8)
            eval( UtilTK.setVars( who, true, varargin{:} ) );

            if resolution == 0 & (pointPwrE ~= 0 | pointPwrI ~= 0)
                resolution = 0.002;
            end
            
            %% add optics
            % laser source
            obj.addLaser( 'src' );
            
            % ITM
            obj.addMirror( 'ITM', 'invRoC', 1/ITMRoC, 'Aperture', ITMAper, 'Thick', 0.2, ...
                'T', T_ITM, 'Lhr', RTL/2, 'Lar', 0, 'removeCurvature', removeCurvature );
            % ETM
            obj.addMirror( 'ETM', 'invRoC', 1/ETMRoC, 'Aperture', ETMAper, 'Thick', 0.2, ...
                'T', T_ETM, 'Lhr', RTL/2, 'resolution', resolution, 'removeTilt', removeTilt, 'removeCurvature', removeCurvature );
            
            % detector behind ETM
            obj.addDetector( 'trans' );
            
            %% define connections among optics
            obj.addProp( '', 'src', 'ITM-AR' );
            obj.addProp( 'FPprop', 'ITM-HR', 'ETM-HR', Lcav);
            obj.addProp( '', 'ETM-AR', 'trans' );
            
          
            %% use the measured maps
            % dat = calcThermal( elph, r, Psub, Pcoat, w, thickness, radius ) 
            %    thermal deformation by absorption using Hello-Vinet formula
            % del = pointAbs( x, y, x0, y0, Pabs, absRadius )
            %    parametrization of surface aberration by point absorber
            %    (x0, y0) : location of point absorber in m
            %    Pabs : amount of absorption in W
            %    multiple absorbers : x0 = [0.02, 0.03], y0 = [-0.01, 0.02], Pabs = [0.04, 0.02]
            % ITM HR surface
             obj.setHRfiles( 'ITM', ITMmap, ...
                 'map + rsq*delRoCI/2 + calcThermal(0, r, 0, PcoatI, wI, 0.2, 0.175 ) + pointAbs(x,y,xPoI, yPoI, pointPwrI)', ...
                 0.16, ITMRoC, 0, ...
                 'delRoCI', delRoCI, 'PcoatI', PcoatI, 'wI', 0.053, 'xPoI', xPoI, 'yPoI', yPoI, 'pointPwrI', pointPwrI );

 
             % ETM HR surface
             obj.setHRfiles( 'ETM', ETMmap, ...
                 'map + rsq*delRoCE/2 + calcThermal(0, r, 0, PcoatE, wE, 0.2, 0.175 ) + pointAbs(x,y,xPoE, yPoE, pointPwrE)', ...
                 0.16, ETMRoC, 0, 'delRoCE', delRoCE, 'PcoatE', PcoatE, 'wE', 0.062, 'xPoE', xPoE, 'yPoE', yPoE, 'pointPwrE', pointPwrE );
        end
    end
end
